﻿#target indesign
#targetengine "getPageRanges"

var script = {
	file: new File( $.fileName ),
	name: 'getPageRanges',
	version: 1.0,
	versionStr: function(){
		return this.version%1 ? this.version.toString() : this.version+'.0';
		},
	insertLabel: function( string ){
		app.insertLabel( this.name, string );
		},
	extractLabel: function(){
		return app.extractLabel( this.name );
		},
	};

var win = new Window("palette", script.name+' v.'+script.versionStr());
win.orientation = 'column';
win.range = win.add( "edittext", undefined, script.extractLabel(),  {multiline: true});
win.range.preferredSize = [400,50];
win.autoCapture = win.add( "checkbox", undefined, "Auto capture" );
win.addStory = win.add( "button", undefined, "Add story" );

win.autoCapture.onClick = function(){
	if( this.value )
		app.activeDocument.addEventListener( 'afterSelectionChanged', addRange );
	else
		app.activeDocument.removeEventListener( 'afterSelectionChanged', addRange );
	}
win.addStory.onClick = addRange;

function addRange(){
	try{
//	alert(getPageRange( "selected story" ));
	var range = getPageRange( "selected story" );
	
	if( range ){
		if( range.indexOf ("-") > 0 ){
			var _pages = range.split("-");
			if( _pages[0] == _pages[1] )
				range = _pages[0];
			}
		win.range.text += ( win.range.text  ? ', ' : '' ) + range;
		}
	}catch(err){ 
			alert(err + ', line ' + err.line );
			}
	}

win.onClose = function(){
	try{
	script.insertLabel(win.range.text);
	app.activeDocument.removeEventListener( 'afterSelectionChanged', addRange );
	}catch(err){ 
			alert(err + ', line ' + err.line );
			}
	}

win.center();
win.show();

function getPageRange( area ){ // return string
	// area = key "current page" | "current spread" | "selected story" | "document" | "selected area"
	try{
		var activePage = app.activeDocument.layoutWindows[0].activePage;
		}catch(err){ 
			var activePage = false;
			}
	switch( area ){
		case 'document'://весь документ
			return getPageFullName( app.activeDocument.pages[0] )+"-"+getPageFullName( app.activeDocument.pages[app.activeDocument.pages.length-1] );
		case 'current page'://текущая страница
			if( app.selection[0] ){
				var page = getPageOrSpreadOfObject(  app.selection[0] );
				if( page && page.constructor == Page )
					return getPageFullName( page );
				}
			if( activePage )
				return getPageFullName( activePage );
			break;
		case 'current spread'://текущий разворот
			if( app.selection[0] ){
				var item = getPageOrSpreadOfObject( app.selection[0] );
				if( page ){
					return item.constructor == Spread ? getPageFullName( item ) :  getPageFullName( item.parent.pages[0] ) +'-'+ getPageFullName( item.parent.pages[1] );;
					}
				}
			if( activePage ){
				if( activePage.parent.pages.length == 2 )//если разворот из двух страниц
					return getPageFullName( activePage.parent.pages[0] ) +'-'+ getPageFullName( activePage.parent.pages[1] );
				else
					return getPageFullName( activePage );
				}
			return false;
		case 'selected story'://выделеная Story
			if( !app.selection[0] ) return false;
			// получаем Story
			var story;
			var obj = app.selection[0];
			try{
				z = 0;
				while( !obj.hasOwnProperty("parentStory") && obj.constructor.name != 'Application' ){
					obj = obj.parent;
					}
				story = obj.parentStory;
				}catch(err){ return false; }
			
			// диапазон между первой и последней страницей Story
			try{
				return getPageFullName( story.textContainers[0].parentPage ) + "-" + getPageFullName( story.textContainers[story.textContainers.length-1].parentPage );
			}catch(err){ alert(err + '\r' + err.line) }
				
			// ======== вычисляет точный диапазон
			
			var pageIndex;
			var pageIndexes = [];
			var indexes = [];
			var textContainers = story.textContainers;
			for( var i = 0; i < textContainers.length; i++ ){
				//будет выдавать ошибки если нет текста (т.е. нет insertionPoints) или если фрейм стоит вне страницы (т.е. на монтаж.столе). все это пропускаем
				//Т.о. в список попадут только фреймы (и патчи) с текстом
				appVersion = parseFloat( app.version );
				if( appVersion < 5 )
					try{ pageIndex = textContainers[i].insertionPoints[0].parentTextFrames[0].parent.documentOffset } catch(err){ continue; }
				else
					try{ pageIndex = textContainers[i].insertionPoints[0].parentTextFrames[0].parentPage.documentOffset } catch(err){ continue; }
				if( indexes[pageIndex] )
					continue;
				pageIndexes.push( pageIndex );
				indexes[pageIndex] = true;
				}
			if( !pageIndexes.length ) return false;
			var pages = app.activeDocument.pages;
			if( pageIndexes.length == 1 ) return getPageFullName( pages[pageIndexes[0]] );
			
			pageIndexes.sort(function(a,b){return (a > b)?1:0});
			var ranges = [ [getPageFullName(pages[pageIndexes[0]])] ];
			try{
			for( var i = 1; i < pageIndexes.length; i++ ){
				if( (pageIndexes[i-1]+1) != pageIndexes[i] ){//если текущая не примыкает к предыдущей - создаем новый range
					ranges.push( [getPageFullName(pages[pageIndexes[i]])] );
					}
				else if( i == (pageIndexes.length - 1) || pageIndexes[i] != (pageIndexes[i+1]-1) )// если текущая примыкает к предыдущей, но не примыкает к следующей, либо это последняя страница - закрываем range
					ranges[ranges.length-1].push( getPageFullName(pages[pageIndexes[i]]) );
				}
			}catch(err){ alert(err + '\r' + err.line) }
			for( var i = 0; i < ranges.length; i++ )
				ranges[i] = ranges[i].join('-');
			return ranges.join(', ');
			break;
		case 'selected area'://выделеная Story
			var SEL = app.selection;
			if( !SEL || !SEL.length || !SEL[0].hasOwnProperty('parentPage') )
				return false;
			var page_indexes = {};
			for( var i = 0; i < SEL.length; i++ ){
				if( !SEL[i].parentPage )
					continue;
				page_indexes[SEL[i].parentPage.documentOffset] = true;
				}
			var indexes = [];
			for( var i in page_indexes )
				indexes.push( i );
			if( !indexes.length )
				return false;
			indexes.sort();
			var pages = app.activeDocument.pages;
			ranges = getPageFullName( pages[indexes[0]] );
			if( indexes.length > 1 )
				ranges = ranges + '-' + getPageFullName( pages[indexes[indexes.length-1]] );
			return ranges;
			break;
		default:
		return false;
		}
	}
function getPageFullName( page ){
	return page.appliedSection.name + page.name;
	}
function getPageOrSpreadOfObject( OBJECT ){ // return Page | Spread | MasterSpread | false
		try{
		// v.2
		// INI
		if( !OBJECT )
			return false;
		if( OBJECT.constructor.name == 'Application' )
			return false;
		if( OBJECT.hasOwnProperty("parentPage") ){
			if( OBJECT.parentPage )
				return OBJECT.parentPage;
			else return false;
			}
		switch(  OBJECT.parent.constructor.name ){
			case 'Page':
				return OBJECT.parent;
			break;
			case 'Spread':
			case 'MasterSpread':
				return OBJECT.parent;
			break;
			default:
				if( OBJECT.hasOwnProperty("parentTextFrames") ){
					if( !OBJECT.parentTextFrames.length ) // если текст вытеснен
						return arguments.callee( OBJECT.parentStory.textContainers[OBJECT.parentStory.textContainers.length-1] );
					else
						return arguments.callee( OBJECT.parentTextFrames[0] );
					}
				return arguments.callee( OBJECT.parent );
			}
		}catch(err){/*alert(err + ', line: '+err.line); */return false; }
		}

